import argparse
from pyratebay.main import search_command, info_command, hot_command
from pyratebay.formatter import format_media_list, format_media_info, format_hot_list

def main() -> None:
    parser = argparse.ArgumentParser(description="pyratebay, a simple command line tool for the pirate bay.")
    subparsers = parser.add_subparsers(dest="command", required=True)

    # search
    parser_search = subparsers.add_parser("search", help="Search torrent by keyword")
    parser_search.add_argument("query", type=str, help="Search keyword")
    parser_search.add_argument("-t", "--type", type=str, default="all", help="Type of content. e.g. movie, tv, music, etc.")
    parser_search.set_defaults(func=search_command, formatter=format_media_list)

    # info
    parser_info = subparsers.add_parser("info", help="Get torrent information by torrent id")
    parser_info.add_argument("tid", type=int, help="torrent id")
    parser_info.set_defaults(func=info_command, formatter=format_media_info)

    # hot
    parser_hot = subparsers.add_parser("hot", help="Get hot torrents")
    parser_hot.add_argument("-t", "--type", type=str, default="movie", help="Type of content. e.g. movie, tv, music, etc.")
    parser_hot.add_argument("-l", "--limit", type=bool, default=False, help="Only show the hot resources within 48h") 
    parser_hot.set_defaults(func=hot_command, formatter=format_hot_list)

    args = parser.parse_args()
    result = args.func(args)
    print(args.formatter(result))

if __name__ == "__main__":
    main()


from pyratebay.data_fetcher import media_search, media_info, hot_media
from pyratebay.models import Media

def search_command(args) -> list[Media]:
    return media_search(args.query, args.type)

def info_command(args) -> Media:
    return media_info(args.tid)

def hot_command(args) -> list[Media]:
    return hot_media(args.type, args.limit)


from dataclasses import dataclass

@dataclass
class Media:
    mid      : str
    title    : str
    desc     : str | None = None
    size     : int | None = None
    seeders  : int | None = None
    leechers : int | None = None
    uploader : str | None = None
    time     : int | None = None
    info_hash: str | None = None


import requests
import json
import time
from urllib import parse
from pyratebay.config import FAKE_HEADERS, API_URL, SEARCH_URL, INFO_URL, HOT_URL, MEDIA_TYP
from pyratebay.models import Media

def media_search(query: str, media_type: str) -> list:
    media_list: list[Media] = []
    search_key: str = parse.quote(query)
    typ: str = MEDIA_TYP.get(media_type) if media_type in MEDIA_TYP else "0"
    url: str = API_URL + SEARCH_URL.format(query=search_key, typ=typ)

    try:
        response: requests.Response = requests.get(url, headers=FAKE_HEADERS)
        data: list = json.loads(response.text)
    except Exception:
        time.sleep(3)
        response: requests.Response = requests.get(url, headers=FAKE_HEADERS)
        data: list = json.loads(response.text)

    for x in data:
        media = Media(
            mid       = x["id"],
            title     = x["name"],
            size      = int(x["size"]) if "size" in x else None,
            seeders   = int(x["seeders"]) if "seeders" in x else None,
            leechers  = int(x["leechers"]) if "leechers" in x else None,
            uploader  = x["username"] if "username" in x else None,
            time      = int(x["added"]) if "added" in x else None,
            info_hash = x["info_hash"] if "info_hash" in x else None,
        )
        media_list.append(media)

    return media_list

def media_info(mid: str) -> Media:
    url: str = API_URL + INFO_URL.format(tid=mid)

    try:
        response: requests.Response = requests.get(url, headers=FAKE_HEADERS)
        data: dict = json.loads(response.text)
    except Exception:
        time.sleep(3)
        response: requests.Response = requests.get(url, headers=FAKE_HEADERS)
        data: dict = json.loads(response.text)

    media = Media(
        mid       = data["id"],
        title     = data["name"],
        desc      = data["descr"] if "size" in data else None,
        size      = data["size"] if "size" in data else None,
        seeders   = data["seeders"] if "seeders" in data else None,
        leechers  = data["leechers"] if "leechers" in data else None,
        uploader  = data["username"] if "username" in data else None,
        time      = data["added"] if "added" in data else None,
        info_hash = data["info_hash"] if "info_hash" in data else None,
    )
    return media

def hot_media(media_type: str, limit: bool) -> list:
    media_list: list[Media] = []

    recent = "_48h" if limit else ""
    typ: str = MEDIA_TYP.get(media_type) if media_type in MEDIA_TYP else "0"
    url:str = API_URL + HOT_URL.format(typ=typ, recent=recent)

    try:
        response: requests.Response = requests.get(url, headers=FAKE_HEADERS)
        data: list = json.loads(response.text)
    except Exception:
        time.sleep(3)
        response: requests.Response = requests.get(url, headers=FAKE_HEADERS)
        data: list = json.loads(response.text)

    for x in data:
        media = Media(
            mid       = x["id"],
            title     = x["name"],
            size      = x["size"] if "size" in x else None,
            seeders   = x["seeders"] if "seeders" in x else None,
            leechers  = x["leechers"] if "leechers" in x else None,
            uploader  = x["username"] if "username" in x else None,
            time      = x["added"] if "added" in x else None,
            info_hash = x["info_hash"] if "info_hash" in x else None,
        )
        media_list.append(media)

    return media_list


API_URL    = "https://apibay.org"
SEARCH_URL = "/q.php?q={query}&cat={typ}"
INFO_URL   = "/t.php?id={tid}"
HOT_URL    = "/precompiled/data_top100{recent}_{typ}.json"
HASH_URL   = "magnet:?xt=urn:btih:{hash}&dn={title}"

MEDIA_TYP = {
    "all"  : "0",
    "movie": "207",
    "tv"   : "208",
    "music": "101",
}

FAKE_HEADERS = {
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/142.0.0.0 Safari/537.36",
    "Referer"   : "https://thepiratebay.org",
}

TR_LIST = [
    'udp://tracker.opentrackr.org:1337',
    'udp://open.stealth.si:80/announce',
    'udp://tracker.torrent.eu.org:451/announce',
    'udp://tracker.bittor.pw:1337/announce',
    'udp://public.popcorn-tracker.org:6969/announce',
    'udp://tracker.dler.org:6969/announce',
    'udp://exodus.desync.com:6969',
    'udp://open.demonii.com:1337/announce',
    'udp://glotorrents.pw:6969/announce',
    'udp://tracker.coppersurfer.tk:6969',
    'udp://torrent.gresille.org:80/announce',
    'udp://p4p.arenabg.com:1337',
    'udp://tracker.internetwarriors.net:1337',
]


import datetime
import textwrap
from urllib import parse
from pyratebay.models import Media
from pyratebay.config import HASH_URL, TR_LIST

def format_media_list(media_list: list[Media]) -> str:
    def list_info(media: Media) -> str:
        return textwrap.dedent(f"""
            [{media.mid}]
            {media.title}
            Size: {fmt_size(media.size)}
            Seeders: {media.seeders}
            Leechers: {media.leechers}
            Uploader: {media.uploader}
            Time: {fmt_time(media.time)}
        """).strip() + "\n"
    
    return "\n".join([list_info(media) for media in media_list])

def format_media_info(media: Media) -> str:
    return textwrap.dedent(f"""
        {media.title}
        Size: {fmt_size(media.size)}
        Seeders: {media.seeders}
        Leechers: {media.leechers}
        Uploader: {media.uploader}
        Time: {fmt_time(media.time)}
        HASH:
        {fmt_hash(media.info_hash, media.title)}

    """).strip() + "\n------\n" + media.desc + "\n------\n"

def format_hot_list(media_list: list[Media]) -> str:
    def list_info(media: Media) -> str:
        return textwrap.dedent(f"""
            [{media.mid}]
            {media.title} - ({fmt_size(media.size)})
            Time: {fmt_time(media.time)}
        """).strip() + "\n"
    
    return "\n".join([list_info(media) for media in media_list])

def fmt_time(ts: int) -> str:
    return datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H:%M:%S')

def fmt_size(size: int) -> str:
    read_size: int = size / 1024 / 1024
    if read_size > 1024:
        return f"{read_size / 1024:.2f} GB"
    else:  
        return f"{read_size:.2f} MB"

def fmt_hash(hash: str, title: str) -> str:
    url:str = HASH_URL.format(hash=hash, title=parse.quote(title))

    for x in TR_LIST:
        url += f"&tr={parse.quote(x, safe="")}"

    return url


from pyratebay.data_fetcher import media_search as search
from pyratebay.data_fetcher import media_info as info 
from pyratebay.data_fetcher import hot_media as hot
from pyratebay.models import Media

__all__ = ["search", "info", "hot", "Media"]
